# TPK4186 - Advanced Tools for Performance Engineering
# BostonHousingDataset.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required Modules
# 2. Loading of training and test sets
# 3. Statistics

# 2. Functions to print results
# 3. Loading of training and test sets
# 4. Logistic regression
# 5. Support Vector Machines
# 6. KNeighborsClassifier
# 7. Naive Bayes
# 8. Decision Trees
# 9. Multi-layer Perceptron


# 1. Required Modules
# -------------------

import numpy
import math
from sklearn import datasets

# 2. Loading of training and test sets
# ------------------------------------

dataSet = datasets.load_boston()
print(dataSet)
sample = dataSet['data']
rewards = dataSet['target']

# 3. Statistics
# -------------

# sepalLengths = sample[:, 0]
# min = sepalLengths.min()
# max = sepalLengths.max()
# mean = sepalLengths.mean()
# print("Sepal lengths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

# sepalWidths = sample[:, 1]
# min = sepalWidths.min()
# max = sepalWidths.max()
# mean = sepalWidths.mean()
# print("Sepal widths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

# petalLengths = sample[:, 2]
# min = petalLengths.min()
# max = petalLengths.max()
# mean = petalLengths.mean()
# print("Petal lengths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

# petalWidths = sample[:, 3]
# min = petalWidths.min()
# max = petalWidths.max()
# mean = petalWidths.mean()
# print("Petal widths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

# 4. Creation of training and test sets.
# --------------------------------------

trainingSample = sample[0:400, :]
trainingRewards = rewards[0:400]

testSample = sample[400:, :]
testRewards = rewards[400:]


# 5. Functions to print results
# -----------------------------

from sklearn import metrics

def PrintResults(actualRewards, predictedRewards):
  print("MAE\t{0:.3e}".format(metrics.mean_absolute_error(actualRewards, predictedRewards)))
  print("RMSE\t{0:.3e}".format(math.sqrt(metrics.mean_squared_error(actualRewards, predictedRewards))))
  print("R^2\t{0:.3e}".format(metrics.r2_score(actualRewards, predictedRewards)))


# 4. Linear Regression
# ----------------------

from sklearn import linear_model

print("Logistic Regression: Start")
model = linear_model.LinearRegression()
model.fit(trainingSample, trainingRewards)
print("Logistic Regression: Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 5. Support Vector Machines
# --------------------------

from sklearn import svm

print("Support Vector Machines: Start")
model = svm.SVR()
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)


# 6. KNeighborsRegressor
# -----------------------

from sklearn.neighbors import KNeighborsRegressor

print(" KNeighborsRegressor: Start")
model =  KNeighborsRegressor(n_neighbors=5)
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 7. Naive Bayes
# --------------

#from sklearn.naive_bayes import GaussianNB
#
#print("Naive Bayes Classifier: Start")
#model = GaussianNB()
#model.fit(trainingSample, trainingRewards)
#print("Training set learned")
#
#predictedRewards = model.predict(testSample)
#PrintResults(testRewards, predictedRewards)

# 8. Decision Trees
# -----------------

from sklearn.tree import DecisionTreeRegressor
print("Decision Trees Regressor: Start")
model = DecisionTreeRegressor()
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 9. Multi-layer Perceptron
# -------------------------

from sklearn.neural_network import MLPRegressor
print("Multi-layer Perceptron: Start")
model = MLPRegressor(max_iter=1000)
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

