# TPK4186 - Advanced Tools for Performance Engineering
# ProcessMonitoring.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required Modules
# 2. Functions to print results
# 3. Loading of training and test sets
# 4. KMeans


# 1. Required Modules
# -------------------

import numpy

# 2. Functions to print results
# -----------------------------

def PrintStatistics(labels):
  counts = [0, 0, 0]
  for i in range(0, len(labels)):
    counts[int(labels[i])] += 1
  for c in range(0, 3):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, counts[c], 100*counts[c]/len(labels)))

def PrintResults(numberOClusters, actualLabels, predictedLabels):
  successes = [0 for _ in range(0, numberOClusters+1)]
  failures = [0 for _ in range(0, numberOClusters+1)]
  actualCounts = [0 for _ in range(0, numberOClusters)]
  predictedCounts = [0 for _ in range(0, numberOClusters)]
  for i in range(0, len(actualLabels)):
    if actualLabels[i]==predictedLabels[i]:
      successes[int(actualLabels[i])] += 1
      successes[numberOClusters] += 1
    else:
      failures[int(actualLabels[i])] += 1
      failures[numberOClusters] += 1
    actualCounts[int(actualLabels[i])]  += 1
    predictedCounts[int(predictedLabels[i])]  += 1
  print("Actual")
  for c in range(0, numberOClusters):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, actualCounts[c], 100*actualCounts[c]/len(actualLabels)))
  print("Predicted")
  for c in range(0, numberOClusters):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, predictedCounts[c], 100*predictedCounts[c]/len(actualLabels)))
  print("Successes")
  for c in range(0,numberOClusters):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, successes[c], 100*successes[c]/actualCounts[c]))
  print("Total:\t{0:d}\t{1:.1f}".format(successes[numberOClusters], 100*successes[numberOClusters]/len(actualLabels)))
  print("Failures")
  for c in range(0, numberOClusters):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, failures[c], 100*failures[c]/actualCounts[c]))
  print("Total:\t{0:d}\t{1:.1f}".format(failures[numberOClusters], 100*failures[numberOClusters]/len(actualLabels)))

# 3. Loading of training and test sets
# ------------------------------------

VERSION = 1
trainingSet = numpy.genfromtxt("trainingSet{0:d}.csv".format(VERSION))
trainingSample = trainingSet[:, :-1]
trainingLabels = trainingSet[:, -1]

testSet = numpy.genfromtxt("testSet{0:d}.csv".format(VERSION))
testSample = testSet[:, :-1]
testLabels = testSet[:, -1]

PrintStatistics(trainingLabels)
PrintStatistics(testLabels)

# 4. KMeans
# ---------

from sklearn.cluster import KMeans

numberOClusters = 2
print("KMeans: Start")
model = KMeans(n_clusters=numberOClusters, random_state=0)
model.fit(trainingSample)
print("Logistic Regression: Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(numberOClusters, testLabels, predictedLabels)

# 5. DBSCAN
# ---------

from sklearn.cluster import DBSCAN

numberOClusters = 2
print("DBSCAN: Start")
model = DBSCAN()
model.fit(trainingSample)
print("DBSCAN: Training set learned")
labels = model.labels_
clusters = dict()
for label in  labels:
  clusters[label] = clusters.get(label, 0) + 1
for cluster in clusters:
  print(str(cluster) + "\t" + str(clusters[cluster]))


#predictedLabels = model.predict(testSample)
#PrintResults(numberOClusters, testLabels, predictedLabels)

