# TPK4186 - Advanced Tools for Performance Engineering
# ProcessMonitoring.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required Modules
# 2. Functions to print results
# 3. Loading of training and test sets
# 4. Logistic regression
# 5. Support Vector Machines
# 6. KNeighborsClassifier
# 7. Naive Bayes
# 8. Decision Trees
# 9. Multi-layer Perceptron


# 1. Required Modules
# -------------------

import numpy
import math

# 2. Functions to print results
# -----------------------------

from sklearn import metrics

def PrintResults(actualRewards, predictedRewards):
  print("MAE\t{0:.3e}".format(metrics.mean_absolute_error(actualRewards, predictedRewards)))
  print("RMSE\t{0:.3e}".format(math.sqrt(metrics.mean_squared_error(actualRewards, predictedRewards))))
  print("R^2\t{0:.3e}".format(metrics.r2_score(actualRewards, predictedRewards)))

def PrintStatistics(rewards):
  print("Reward {0:.3e}:\t{1:.3e}\t{2:.3e}".format(rewards.min(), rewards.max(), rewards.mean()))

# 3. Loading of training and test sets
# ------------------------------------

VERSION = 5
trainingSet = numpy.genfromtxt("trainingSet{0:d}.csv".format(VERSION))
trainingSample = trainingSet[:, :-1]
trainingRewards = trainingSet[:, -1]

testSet = numpy.genfromtxt("testSet{0:d}.csv".format(VERSION))
testSample = testSet[:, :-1]
testRewards = testSet[:, -1]

PrintStatistics(trainingRewards)
PrintStatistics(testRewards)

# 4. Linear Regression
# ----------------------

from sklearn import linear_model

print("Logistic Regression: Start")
model = linear_model.LinearRegression()
model.fit(trainingSample, trainingRewards)
print("Logistic Regression: Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 5. Support Vector Machines
# --------------------------

from sklearn import svm

print("Support Vector Machines: Start")
model = svm.SVR()
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)


# 6. KNeighborsRegressor
# -----------------------

from sklearn.neighbors import KNeighborsRegressor

print(" KNeighborsRegressor: Start")
model =  KNeighborsRegressor(n_neighbors=5)
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 7. Naive Bayes
# --------------

#from sklearn.naive_bayes import GaussianNB
#
#print("Naive Bayes Classifier: Start")
#model = GaussianNB()
#model.fit(trainingSample, trainingRewards)
#print("Training set learned")
#
#predictedRewards = model.predict(testSample)
#PrintResults(testRewards, predictedRewards)

# 8. Decision Trees
# -----------------

from sklearn.tree import DecisionTreeRegressor
print("Decision Trees Regressor: Start")
model = DecisionTreeRegressor()
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

# 9. Multi-layer Perceptron
# -------------------------

from sklearn.neural_network import MLPRegressor
print("Multi-layer Perceptron: Start")
model = MLPRegressor(hidden_layer_sizes=(200, 200), max_iter=1000)
model.fit(trainingSample, trainingRewards)
print("Training set learned")

predictedRewards = model.predict(testSample)
PrintResults(testRewards, predictedRewards)

