# TPK4186 - Advanced Tools for Performance Engineering
# TrawlersWay.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import random

# 1. FishZones
# ------------

class FishingZone:
  def __init__(self, numberOfRows, numberOfColumns):
    self.numberOfRows = numberOfRows
    self.numberOfColumns = numberOfColumns
    self.grid = [[random.random() for column in range(0, self.numberOfColumns)] for row in range(0, self.numberOfRows)]

# 2. Recursive Algorithm
# ----------------------

def RecursiveBestRoute(fishingZone):
  bestReward = 0
  bestRoute = None
  for row in range(0, fishingZone.numberOfRows):
    (route, reward) = RecursiveExploration(row, 0, fishingZone)
    if reward>bestReward:
      bestReward = reward
      bestRoute = route
  return (bestRoute, bestReward)

def RecursiveExploration(row, column, fishingZone):
  reward = fishingZone.grid[row][column]
  if column==fishingZone.numberOfColumns-1:
    route = [(row, column)]
  else:
    if row>0:
      (northRoute, northReward) = RecursiveExploration(row-1, column+1, fishingZone)
    else:
      northReward = 0
    (eastRoute, eastReward)  = RecursiveExploration(row, column+1, fishingZone)
    if row<fishingZone.numberOfRows-1:
      (southRoute, southReward) = RecursiveExploration(row+1, column+1, fishingZone)
    else:
      southReward = 0
    if northReward>eastReward:
      if northReward>southReward:
        reward += northReward
        route = northRoute
      else:
        reward += southReward
        route = southRoute
    else:
      if eastReward>southReward:
        reward += eastReward
        route = eastRoute
      else:
        reward += southReward
        route = southRoute
    route.insert(0, (row, column))
  return (route, reward)

# 3. Dynamic Programming Algorithm
# --------------------------------

def DynamicProgrammingBestRoute(fishingZone):
  bestRouteGrid = [[None for column in range(0, fishingZone.numberOfColumns)] for row in range(0, fishingZone.numberOfRows)]
  DynamicProgrammingExploration(fishingZone, bestRouteGrid)
  bestReward = 0
  bestRoute = None
  for row in range(0, fishingZone.numberOfRows):
    (route, reward) = bestRouteGrid[row][0]
    if reward>bestReward:
      bestReward = reward
      bestRoute = route
  return (bestRoute, bestReward)

def DynamicProgrammingExploration(fishingZone, bestRouteGrid):
  for column in range(fishingZone.numberOfColumns-1, -1, -1):
    for row in range(0, fishingZone.numberOfRows):
      reward = fishingZone.grid[row][column]
      route = [(row, column)]
      if column<fishingZone.numberOfColumns-1:
        if row>0:
          (northRoute, northReward) = bestRouteGrid[row-1][column+1]
        else:
          northReward = 0
        (eastRoute, eastReward) = bestRouteGrid[row][column+1]
        if row<fishingZone.numberOfRows-1:
          (southRoute, southReward) = bestRouteGrid[row+1][column+1]
        else:
          southReward = 0
        if northReward>eastReward:
          if northReward>southReward:
            reward += northReward
            route.extend(northRoute)
          else:
            reward += southReward
            route.extend(southRoute)
        else:
          if eastReward>southReward:
            reward += eastReward
            route.extend(eastRoute)
          else:
            reward += southReward
            route.extend(southRoute)
      bestRouteGrid[row][column] = (route, reward)

# 4. Caching Algorithm
# --------------------

def CachingBestRoute(fishingZone):
  bestReward = 0
  bestRoute = None
  cache = [(-1, -1, [], -1) for i in range(0, 1000)]
  for row in range(0, fishingZone.numberOfRows):
    (route, reward) = CachedExploration(row, 0, cache, fishingZone)
    if reward>bestReward:
      bestReward = reward
      bestRoute = route
  return (bestRoute, bestReward)

def CachedExploration(row, column, cache, fishingZone):
  reward = fishingZone.grid[row][column]
  route = [(row, column)]
  if column==fishingZone.numberOfColumns-1:
    return (route, reward)
  hashCode = (13*row + 7*column) % len(cache)
  (cachedRow, cachedColumn, cachedRoute, cachedReward) = cache[hashCode]
  if cachedRow==row and cachedColumn==column:
    return (cachedRoute, cachedReward)
  if row>0:
    (northRoute, northReward) = CachedExploration(row-1, column+1, cache, fishingZone)
  else:
    northReward = 0
  (eastRoute, eastReward) = CachedExploration(row, column+1, cache, fishingZone)
  if row<fishingZone.numberOfRows-1:
    (southRoute, southReward) = CachedExploration(row+1, column+1, cache, fishingZone)
  else:
    southReward = 0
  if northReward>eastReward:
    if northReward>southReward:
      reward += northReward
      route.extend(northRoute)
    else:
      reward += southReward
      route.extend(southRoute)
  else:
    if eastReward>southReward:
      reward += eastReward
      route.extend(eastRoute)
    else:
      reward += southReward
      route.extend(southRoute)
  cache[hashCode] = (row, column, route, reward)
  return (route, reward)

# 5. Tests
# --------

fishingZone = FishingZone(10, 15)

(route, reward) = RecursiveBestRoute(fishingZone)
print(reward)
print(route)

(route, reward) = DynamicProgrammingBestRoute(fishingZone)
print(reward)
print(route)

(route, reward) = CachingBestRoute(fishingZone)
print(reward)
print(route)
